game = {
	debug: false,
	state: "loading",
	time: 0,
	MAX_CREW: 10,
	init: function(){
		//init first chunk

		game.difficultyFactor = 1;

		game.objects = {};
		map.chunk(0,0);

		game.sidebarPerson = null;

		game.previousBig = null;
		game.tab = "crew";
		game.bigAngle = 0;
		game.items = [];

		//camp chunk
		var campAngle = Math.random()*2*Math.PI;
		var campDistance = util.randRange(60,80);
		var campX = Math.cos(campAngle)*campDistance;
		var campY = Math.sin(campAngle)*campDistance;
		game.campChunkX = Math.floor(campX/CHUNK_SIZE);
		game.campChunkY = Math.floor(campY/CHUNK_SIZE);

		game.time = util.randRange(60*6,60*6);

		game.lastSleep = game.time;

		game.globalCounter = 0;

		game.tile = map.tile(0,0);

		var lastName = initOptions.lastName;
		var offspringGender = initOptions.offspringGender;
		var offspringName = initOptions.offspringName;

		var parentGender = game.randomGender();

		game.player = new Person({
			gender:parentGender,
			age: util.randRange(25,45),
			name: game.getName(parentGender, lastName).toUpperCase(),
			badTraits: false
		});
		game.player.isPlayer = true;
		game.offspring = new Person({
			gender:offspringGender,
			age: util.randRange(4,6),
			name: offspringName.toUpperCase(),
			badTraits: false
		});
		game.offspring.isOffspring = true;

		game.player.pc = true;
		game.offspring.pc = true;

		game.crew = [game.player, game.offspring];

		for(var i=0;i<game.crew.length;i++){
			for(var j=0;j<game.crew[i].traits.length;j++){
				game.crew[i].traits[j].known = true;
			}
		}

		people.setupFamily(game.crew);


		stash.init();

		game.crew[0].equipItem(new ItemKnife());
		people.renderPerson(game.crew[0]);

		game.timeTint = game.getTimeTint();

		game.hoverTile = camera.getTile();
	},
	isDriving: function(){
		return game.crew.length > 0 && stash.gas >= game.gasCost() && camera.getTile().road;
	},
	isNight:function(){
		var hour = game.getTimeOfDay()/60;
		if(hour>=20 || hour <= 4){
			return true;
		}else{
			return false;
		}
	},
	isRoughlyNight:function(){
		var hour = game.getTimeOfDay()/60;
		if(hour>=18 || hour <= 6){
			return true;
		}else{
			return false;
		}
	},
	getTimeOfDay: function(){
		return game.time % (60*24);
	},
	getTimeDescription:function(){
		var t = game.getTimeOfDay();
		
		var hour = Math.floor((t/60))%12;
		if(hour==0){
			hour+=12;
		}
		
		if(hour<10){
			hour = hour;
		}
		
		var minute = Math.floor(t % 60);
		if(minute<10){
			minute = "0"+minute;
		}
		
		var description = "";
		description+=hour+":"+minute;
		if(t>=60*12){
			description+="pm";
		}else{
			description+="am";
		}
		
		return description;
	},
	win:function(){
		var content = "";

		content += "<p>Congratulations! You found the refugee camp and saved "+game.offspring.name+".</p>";

		content += '<div class="button" button-id="new-game">New Game</div> ';

		setTimeout(function(){
			menu.showMenu(
				"We can start over here...",
				content,
				false,
				"game-over"
			);
		},1000);

		game.state = "won";
	},
	closeSidebarDetail:function(){
		game.sidebarPerson = null;
		$("#sidebar-detail").hide();
	},
	destroy: function(obj){
		delete game.objects[obj.id];
	},
	getVillainName: function(gender){
		if(gender == "female" || Math.random()<.7){
			return util.pick(game.lastNames);
		}else{
			return "The "+util.pick(game.villains);
		}
	},
	randomGender:function(){
		return util.pick(["male","female"]);
	},
	getName: function(gender, lastName){
		if(gender == "female"){
			var firstName = game.firstNames[util.randRange(400,799)];
		}else{
			var firstName = game.firstNames[util.randRange(0,399)];
		}
		if(lastName === undefined){
			var lastName = util.pick(game.lastNames);
		}

		return firstName + " " + lastName;
	},
	tryMove:function(dx,dy){
		var newTile = map.tile(game.tile.x+dx,game.tile.y+dy);

		game.tile = newTile;

		camera.setPosition(newTile);
		game.hoverTile = map.tile(camera.x,camera.y);
		game.renderSelector();

		if(!newTile.explored){
			newTile.explore();
		}

		var dt;
		if(game.isDriving()){
			//4 people per car
			stash.gas -= game.gasCost();
			var dt = 0.05; // ~3m
		}else{
			//good speed (~10) = half hour to move a tile
			var dist = util.distance(0,0,dx,dy);
			dt = dist*0.5*10/people.getCrewSpeed();
		}

		game.tick(dt);
	},

	gasCost:function(){
		return Math.ceil(game.crew.length/4);
	},


	getTimeTint:function(){
		if(uglyMode){
			return 0xffffff;
		}

		var timeTint = 0;
		var hour = game.getTimeOfDay()/60;
		if(hour <= 3 || hour >= 21){
			timeTint = 0;
		}else if(hour <= 7){
			timeTint = (hour - 3)/4;
		}else if(hour >= 17){
			timeTint = (21 - hour)/4;
		}else{
			timeTint = 1;
		}


		var r = Math.floor(255*(timeTint*.75+.25)) * 256*256;
		var g = Math.floor(255*(timeTint*.75+.25)) * 256;
		var b = Math.floor(255*(timeTint*.25+.75));

		if(game.state == "game-over"){
			return 0xff0000;
		}

		return r+g+b;
	},

	awakeDuration: function(){
		return (game.time - game.lastSleep)/60;
	},

	tick: function(dt, sleep){
		if(dt===undefined){
			dt = 1;
		}

		console.log(Math.floor(dt*60)+"minutes!");

		game.time += dt*60;

		if(sleep){
			game.lastSleep = game.time;
		}

		var clan = map.getClan();
		if(!clan.revealed && Math.random()< map.getClanRevealChance()){
			clan.revealed = true;
			menu.toast('REVEALED "'+clan.name.toUpperCase()+'!"');
		}

		for(var i=0;i<game.crew.length;i++){
			game.crew[i].update(dt, sleep);
		}

		window.requestAnimationFrame(draw);
		game.drawUI();

		if(game.state === "running" && !sleep){
			events.doEvent(dt);
		}

		camera.getTile().increaseZombies();

		game.difficultyFactor += .03;
		if(game.difficultyFactor>4){
			game.difficultyFactor = 4;
		}
	},
	violentOutcome:function(zombie,weaponType){
		if(game.crew.length === 0)return;

		var crewMember = util.pick(game.crew, true);
		//save your kid
		if(crewMember.isOffspring && game.crew[0].equals(game.player)){
			crewMember = game.player;
		}

		game.violentOutcomeOnPerson(crewMember, zombie, weaponType)

	},
	violentOutcomeOnPerson(crewMember, zombie, weaponType){
		if(crewMember.dead) return;

		var hitChance;

		if(weaponType == "rifle"){
			hitChance = .5;
		}else if(weaponType == "handgun"){
			hitChance = .75;
		}else if(weaponType == "bat"){
			hitChance = .9;
		}else{
			hitChance = 1;
		}

		if(crewMember.age<10){
			hitChance *= .5;
		}


		var slot = crewMember.randomSlot();
		if(zombie){
			if(!slot.item){
				if(crewMember.isOffspring){
					crewMember.die();
				}else{
					menu.toast(crewMember.name +" has been BITTEN!");
					//bitten too many times in the same spot
					if(slot.bitten){
						crewMember.die();
					}else{
						slot.bite();
					}
				}
			}
		}else{
			if(slot.type === "leg"){
				if(!crewMember.hasTrait("Limp")){
					crewMember.traits.push(new AttributeLimp());
					crewMember.speed *= .9;
				}
			}

			if(weaponType == "rifle" || weaponType == "handgun"){
				if(slot.type === "head"){
					crewMember.die();
				}else if(slot.type === "torso" && Math.random()<.5){
					crewMember.die();
					//this.traits.push(new TraitGeneric(this.gender, false));
				}else{
					if(Math.random()<.5){
						if(!crewMember.hasTrait("Bleeding")){
							crewMember.traits.push(new TraitBleeding());
							menu.toast(crewMember.name +" is bleeding.");
						}
					}else if(!slot.breakable){
						slot.break(crewMember);
					}
				}
			}else{
				//today: *maybe* check item type somehow
				if(!slot.item){
					if(Math.random()<.5 || !slot.breakable){
						if(!crewMember.hasTrait("Bleeding")){
							crewMember.traits.push(new TraitBleeding());
							menu.toast(crewMember.name +" is bleeding.");
						}
					}else{
						slot.break(crewMember);
					}
				}
			}
		}
	},
	drawUI:function(){
		if(game.tab === "crew"){
			people.renderCrew();
		}else{
			stash.render();
		}

		if(game.sidebarPerson != null){
			people.renderPerson(game.sidebarPerson);
		}

		//leftbar
		var html = "";
		html += "<strong>CURRENT LOCATION</strong>";
		if(game.isDriving()){
			html += ' <span style="color:white;"><strong>[DRIVING]</strong></span>';
		}
		html += '<br>';
		html += camera.getTile().getLongDescription();
		html += "<br>";
		html += "<strong>Time: </strong>" + game.getTimeDescription();
		$("#left-bar-current").html(html);
	},
	drawNearby: function(){
		if(game.hoverTile && !game.hoverTile.equals(camera.getTile())){
			var html = "";
			html += "<strong>NEARBY LOCATION</strong><br>";
			html += game.hoverTile.getLongDescription();
			$("#left-bar-examine").html(html);
		}else{
			$("#left-bar-examine").html("");
		}
	},
	setHoverTile:function(x,y){
		if(game.state === "running" && !menu.menuOpen){
			game.hoverTile = map.tile(
				Math.floor(x / camera.tileSize)-camera.getOffsetX(),
				Math.floor(y / camera.tileSize)-camera.getOffsetY()
			);

			if(x == undefined && y == undefined){
				game.hoverTile = map.tile(camera.x,camera.y);
			}

			if(Math.abs(game.hoverTile.x-camera.x)>camera.centerOffsetX || 
			Math.abs(game.hoverTile.y-camera.y)>camera.centerOffsetY){
				game.hoverTile = map.tile(camera.x,camera.y);
			}
			game.renderSelector();
		}
	},
	renderSelector:function(){
		if(game.selectorSprite){
			var tileX = camera.tileSize*(game.hoverTile.x + camera.getOffsetX());
			var tileY = camera.tileSize*(game.hoverTile.y + camera.getOffsetY());
			game.selectorSprite.position.x = tileX;
			game.selectorSprite.position.y = tileY;
			game.selectorSprite.scale = new PIXI.Point(
				camera.tileSize/256,
				camera.tileSize/256
			);
			overlayRenderer.render(overlayStage);
		}
	}

	/*var neighbors = camera.getTile().getAllNeighbors();
			for(var i=0;i<neighbors.length;i++){
				var tileX = camera.tileSize*(neighbors[i].x + camera.getOffsetX()+.5);
				var tileY = camera.tileSize*(neighbors[i].y + camera.getOffsetY()+.5);

				var num = neighbors[i].getZombies();

				var text = new PIXI.Text(num,{
					fontFamily:"Arial",
					fontSize: .5*(camera.tileSize)+"px",
					fill:"#00aa00",
					strokeThickness: (camera.tileSize)/16,
					stroke: "black"
				});
				text.position.x = tileX;
				text.position.y = tileY;
				text.anchor.set(0.5);
				overlayStage.addChild(text);
			}
//'fontFamily','fontSize',fontStyle','fontVariant' and 'fontWeight' properties from now on

			//*/
}