class Slot{
	constructor(person, type, removable, bitable, side, children, breakable){
		this.item = null;
		this.removed = false;
		if(children===undefined){
			this.children = [];
		}else{
			this.children = children;
		}
		this.removable = removable;
		this.type = type;

		if(bitable === undefined){
			this.bitable = true;
		}else{
			this.bitable = bitable;
		}

		if(breakable === undefined){
			breakable = false;
		}
		this.breakable = breakable;

		this.bitten = false;
		this.broken = false;

		this.dead = false;

		if(side){
			this.side = side;
			this.name = side +" "+this.type;
		}else{
			this.name = this.type;
		}

		this.zombie = false;

		person.slots[this.name] = this;

		this.id = ++game.globalCounter;
		game.objects[this.id] = this;
	}

	amputate(person){
		this.removed = true;

		var visits = [this];
		while(visits.length>0){
			var slot = visits.pop();
			visits = visits.concat(slot.children);
			if(slot.item){
				person.removeItem(slot.item);
			}
			slot.removed = true;
		}

		var infectionChance = .5;
		if(people.crewHasTrait("Surgery")){
			infectionChance = .05;
		}

		//todo: surgery
		if(Math.random()<infectionChance){
			if(!person.hasTrait("Infection")){
				person.traits.push(new TraitInfection());
				menu.toast(person.name + " has developed an infection.");
			}
		}
	}

	break(person){
		this.broken = true;
		this.brokenCounter = util.randRange(50,60);

		var visits = [this];
		while(visits.length>0){
			console.log(visits)
			var slot = visits.pop();
			visits = visits.concat(slot.children);
			if(slot.item){
				person.removeItem(slot.item);
			}
		}
	}

	bite(){
		this.bitten = true;
		this.bittenCounter = util.randRange(10, 20);
	}

	getHtml(){
		var html = "";
		html += '<tr>';
			html += '<td>';
				html += this.name.toUpperCase();
				if(this.bitten){
					html += '<div style="font-weight: bold;color:#00bb22;">(bitten)<div>';
				}
				if(this.broken){
					html += '<div style="font-weight: bold;color:#999;">(disabled)<div>';
				}
			html += '</td>';
			html += '<td>';
				if(this.item != null){
					html += this.item.name;
				}
			html += '</td>';
			html += '<td>';
				if(this.item != null){
					html += '<div class="button" button-id="remove-item" object-id="'+this.item.id+'">Remove</div><br>';
				}else{
					if(this.removable){
						html += '<div class="button amputate-button" button-id="amputate" object-id="'+this.id+'">Amputate</div>';
					}
				}
			html += '</td>';
		html += '</tr>';

		//todo: buttons: BREAK/AMPUTATE
		//todo: infected/broke status

		return html;
	}
}

class Person{
	constructor(options){
		//options: isVillain, gender, age, name, race

		if(options.gender===undefined){
			options.gender = game.randomGender();
		}
		if(options.age ===undefined){
			var r = Math.random();
			if(r<.05){
				options.age = util.randRange(61,90);
			}else if(r<.15){
				options.age = util.randRange(5,17);
			}else{
				options.age = util.randRange(18,60);
			}
		}
		if(options.isVillain === undefined){
			options.isVillain = false;
		}
		if(options.isVillain){
			options.age = util.randRange(30,65);
		}
		if(options.name === undefined){
			if(options.isVillain){
				options.name = game.getVillainName(options.gender);
			}else{
				options.name = game.getName(options.gender);
			}
		}
		if(options.race == undefined){
			options.race = traits.getRace();
		}
		if(options.religion == undefined){
			if(options.age > 10 && Math.random()<.75){
				options.religion = traits.getWeightedReligion();
			}else{
				options.religion = null;
			}
		}
		if(options.badTraits == undefined){
			options.badTraits = true;
		}

		this.gender = options.gender;
		this.age = options.age;
		this.name = options.name;
		this.race = options.race;
		this.religion = options.religion;

		this.physicalDescription = this.getPhysicalDescription(options.gender,options.age);

		this.isPlayer = false;
		this.isOffspring = false;
		this.pc = false;

		this.satiety = util.randRange(30,50);

		//TRAITS
		this.traits = [];
		this.traits.push(this.race);
		if(this.religion){
			this.traits.push(this.religion);
		}
		this.traits.push(new TraitGeneric(this.gender, false));
		if(options.badTraits){
			if(Math.random()<.4 && this.age >= 14){
				this.traits.push(new HatesPeople(this));
			}
			if(Math.random()<.4 && this.age >= 18){
				this.traits.push(traits.getBadAttribute());
			}
		}
		if(Math.random()<.666 && this.age >= 18){
			this.traits.push(traits.getSkill());
		}

		util.shuffle(this.traits);
		//shuffle

		this.items = [];

		this.slots = {};
		var head = new Slot(this, "head", false);

		var rightWeapon = new Slot(this, "weapon", false, false, "right");
		var rightHand = new Slot(this, "hand", true, true, "right", [rightWeapon], true);
		var rightArm  = new Slot(this, "arm", true, true, "right", [rightHand], true);

		var leftWeapon = new Slot(this, "weapon", false, false, "left");
		var leftHand = new Slot(this, "hand", true, true, "left", [leftWeapon], true);
		var leftArm  = new Slot(this, "arm", true, true, "left", [leftHand], true);

		var rightFoot = new Slot(this, "foot", true, true, "right");
		var leftFoot = new Slot(this, "foot", true, true, "left");

		var rightLeg = new Slot(this, "leg", true, true,  "right", [rightFoot]);
		var leftLeg = new Slot(this, "leg", true, true,  "left", [leftFoot]);

		var torso = new Slot(this, "torso", false, false, null, [
			head, rightArm, leftArm, rightLeg, leftLeg
		]);

		if(this.age >= 20){
			var baseSpeed = 10 - .03*Math.pow(this.age-20, 1.3);
		}else{
			var baseSpeed = 10 - .02*Math.pow(20-this.age, 1.6);
		}
		this.speed = Math.max(1,baseSpeed * 1);

		//todo: TEST THIS
		if(this.hasTrait("Limp")){
			this.speed *= .7;
		}

		this.id = ++game.globalCounter;
		game.objects[this.id] = this;

		this.emotions = {};
	}

	getSpeed(){
		var hobbleFactor = 0.25 * this.countWorkingLowerAppendages();
		return Math.max(1,
			Math.floor(hobbleFactor*this.speed*10)/10
		);
	}

	countWorkingLowerAppendages(){
		return this.findWorkingSlots().filter(function(s){
			return s.type === "leg" || s.type === "foot";
		}).length;
	}

	exile(){
		util.remove(game.crew,this);
		if(this.isPlayer){
			menu.toast("You decide that leaving is the only way.");
		}else{
			menu.toast(this.name + " has been exiled, never to return again.");
		}
		game.closeSidebarDetail();

		people.emotionMod(this,game.crew[0],50, undefined, undefined, true);
	}

	kill(){
		util.remove(game.crew,this);
		if(this.isPlayer){
			menu.toast("You can't take the oppressing anxiety, the neverending worry. It's all too much.");
		}else{
			menu.toast("You kill " +this.name +".");
		}
		game.closeSidebarDetail();

		people.emotionMod(this,game.crew[0],100, undefined, undefined, true);

		//YOU GET TO KEEP ITEMS
		var that = this;
		this.findExistingSlots().filter(function(s){
			if(s.item){
				that.removeItem(s.item);
			}
		});
	}

	goCrazy(target){
		menu.toast("In a fit of rage, "+this.name +" attacks "+target.name);
		this.emotions[target.id] = Math.min(75, this.emotions[target.id]);

		var weaponConvert = {
			"long-reach-gun": 'rifle',
			"short-reach-gun": 'handgun',
			"long-reach-melee": 'bat',
			"short-reach-melee": 'knife'
		}

		var weaponSlots = 0;

		var slots = this.findWorkingSlots();
		for(var i=0;i<slots.length;i++){
			var s = slots[i];
			if(s.item && s.item.weaponType){
				game.violentOutcomeOnPerson(
					target,
					false,
					weaponConvert[s.item.weaponType]
				);
				weaponSlots++;
			}
		}

		if(weaponSlots == 0){
			game.violentOutcomeOnPerson(
				target,
				false,
				'knife'
			);
		}
	}



	die(reason){
		this.dead = true;
		util.remove(game.crew,this);
		if(reason){
			menu.toast(this.name + " has died of "+reason+"!");
		}else{
			menu.toast(this.name + " has died!");
		}

		people.emotionMod(this,game.crew[0],50, undefined, undefined, true);



		if(this.isOffspring){
			//menu.closeMenu();

			var content = "";

			content += "<p>This world was too dangerous. Your child "+game.offspring.name+" has perished.</p>";

			content += '<div class="button" button-id="new-game">New Game</div> ';

			setTimeout(function(){
				menu.showMenu(
					"Game Over",
					content,
					false,
					"game-over"
				);
			},1500);

			game.state = "game-over";

			window.requestAnimationFrame(draw);
			game.drawUI();
		}
	}

	removeItem(item){
		var foundItem = false;
		this.findAllSlots().forEach(function(s){
			if(s.item && s.item.equals(item)){
				foundItem = true;
				s.item = null;
			}
		});

		if(foundItem){
			stash.addItem(item);
		}
	}

	fullyEquip(){
		var numEquipped = 0;
		for(var k=stash.items.length-1;k>=0;k--){
			if(this.equipItem(stash.items[k], false)){
				numEquipped++;
			}
		}
		if(numEquipped==0){
			menu.toast("Nothing to equip.");
		}else{
			people.emotionMod(game.sidebarPerson,game.crew[0], -3*numEquipped, true);
		}
	}

	equipItem(item, logOutput){
		if(logOutput === undefined){
			logOutput = true;
		}

		//get all slots which are not broken
		var equipped = false;

		if(item.slotType === "weapon"){
			var baseSlots = this.findWorkingSlots();
		}else{
			var baseSlots = this.findExistingSlots();
		}

		var equippableSlots = baseSlots.filter(function(s){
			return (s.type === item.slotType && s.item==null);
		});

		if(equippableSlots.length < item.slotCount){
			if(logOutput){
				menu.toast("Can't equip the "+item.name);
			}
		}else{
			for(var i=0;i<item.slotCount;i++){
				equippableSlots[i].item = item;
			}

			//E.G. jacket goes on torso and arms
			baseSlots.forEach(function(s){
				if(item.additionalSlotTypes.indexOf(s.type) >-1){
					if(s.item!=null && s.item !==item){
						this.removeItem(s.item);
					}else{
						s.item = item;
					}
				}
			});

			stash.removeItem(item);

			equipped = true;
		}

		return equipped;
	}

	//doesn't return "weapon"
	randomSlot(){
		var slots = this.findExistingSlots().filter(function(t){
			return t.type != "weapon";
		});
		return util.pick(slots,true);
	}
	findWorkingSlots(){
		return this.findSlots(this.slots['torso'], [], {
			broken: true,
			removed: true
		});
	}
	findExistingSlots(){
		return this.findSlots(this.slots['torso'], [], {
			removed: true
		});
	}
	findAllSlots(){
		return this.findSlots(this.slots['torso'], [], {});
	}
	findSlots(slot, workingSlots, options){
		if((!options.removed || !slot.removed) &&
		(!options.broken || !slot.broken)){
			workingSlots.push(slot);
			for(var i=0;i<slot.children.length;i++){
				var child = slot.children[i];
				this.findSlots(child, workingSlots, options);
			}
		}
		return workingSlots;
	}

	update(dt,sleep){
		this.updateEmotions(sleep);

		if(sleep){
			this.satiety -= dt*.333;
		}else{
			if(this.hasTrait("High Metabolism")){
				this.satiety -= dt*1.333;
			}else{
				this.satiety -= dt*.666;
			}
		}

		if(this.hasTrait("Illness") && Math.random()<(dt/48)){
			if(Math.random()>.5){
				if(!this.hasTrait("Bleeding")){
					this.traits.push(new TraitBleeding());
					menu.toast(this.name +" is bleeding.");
				}
			}else{
				if(!this.hasTrait("Infection")){
					this.traits.push(new TraitInfection());
					menu.toast(this.name +" is infected.");
				}
			}
		}

		if(this.satiety <= 0){
			if(this.hasTrait("Cannibal")){
				menu.toast(this.name+" eats "+game.crew[0].name+"!");
				game.crew[0].die();
				this.satiety = 100;
			}else{
				this.die("starvation");
				return;
			}
		}else if(this.satiety < 10){
			menu.toast(this.name + " is STARVING!");
		}

		for(var i=0;i<this.traits.length;i++){
			var trait = this.traits[i];
			if(!trait.known){
				if(trait.name != "Introvert" || Math.random()<.5){
					trait.knownCounter--;
				}

				if(trait.knownCounter<=0){
					trait.known = true;
					menu.toast(this.name+" reveals trait:<br>"+trait.getHtml());
				}
			}
			if(trait.shitFan){
				trait.shitFanCounter-= dt;
				if(trait.shitFanCounter<=0){
					trait.shitHitFan(this);
				}
			}
		}

		var slots = this.findExistingSlots();
		for(var i=0;i<slots.length;i++){
			if(slots[i].broken){
				slots[i].brokenCounter-=dt;
				if(slots[i].brokenCounter<=0){
					slots[i].broken = false;
				}
			}
			if(slots[i].bitten){
				slots[i].bittenCounter-=dt;
				if(slots[i].bittenCounter<=0){
					this.zombie = true;
				}
			}
		}
	}

	updateEmotions(sleep){
		for(var i=0;i<game.crew.length;i++){
			var person = game.crew[i];
			var emotion = this.getEmotion(person);

			if(sleep){
				//no more hatred after sleep
				if(emotion>50){
					emotion = Math.min(
						65,
						Math.max(0, emotion - 15)
					);
				}
			}else{
				//no way to control feelings against child, so don't let this happen
				//while the parent is alive
				if(!(person.isOffspring && game.crew[0].isPlayer)){
					var awake = game.awakeDuration();

					if(this.hasTrait("Temper")){
						emotion += util.randRange(-2,7);
					}else{
						if(awake > 24){
							emotion += util.randRange(-2,6);
						}else if(awake > 16){
							emotion += util.randRange(-2,4);
						}else if(awake > 8){
							emotion += util.randRange(-2,2);
						}else{
							emotion += util.randRange(-3,2);
						}
					}
				}
			}

			this.emotions[person.id] = emotion;

			if(emotion > 80){
				//alert("real hatred "+this.name+" "+person.name);
			}
		}
	}

	getEmotion(target){
		if(this.emotions.hasOwnProperty(target.id)){
			return this.emotions[target.id];
		}else{
			if(this.equals(target)){
				if(this.hasTrait("Narcissist")){
					var emotion = -1000;
				}else if(this.hasTrait("Suicidal")){
					var emotion = util.randRange(50,70);
				}else{
					var emotion = util.randRange(0,20);
				}
			}else{
				var emotion = 50;
			}
			
			if(this.hates(target)){
				emotion += 20;
			}

			this.emotions[target.id] = emotion;
			return emotion;
		}
	}

	hates(target){
		for(var i=0;i<this.traits.length;i++){
			var trait = this.traits[i];
			if(trait.name === "hatred" && target.hasTrait(trait.target)){
				return true;
			}
		}
		return false;
	}

	getEmotionHtml(){
		var html = "";


		var emotions = [];
		for(var i=0;i<game.crew.length;i++){
			var person = game.crew[i];
			var emotion = this.getEmotion(person);

			if(!person.equals(this) ||
			(person.hasTrait("Suicidal") ||
			person.hasTrait("Narcissist"))){
				if(emotion >= 60 || emotion <= 40){
					emotions.push({
						person: person,
						emotion: emotion
					});
				}
			}
		}
		
		html += '<table class="emotions-table">';
		if(emotions.length > 0 && !this.pc){
			emotions.sort(function(a,b){
				return b.emotion - a.emotion;
			});

			html += '<tr><td colspan="2"><h4>Feelings</h4></td></tr>';
			for(var i=0;i<emotions.length;i++){
				html += '<tr>';
				if(emotions[i].person.equals(this)){
					html += '<td>Self</td>';
				}else{
					html += '<td>'+emotions[i].person.name+'</td>';
				}

				if(emotions[i].emotion>=75){
					var emotionColor = '#cc0000';
					var emotionClass = "horrible-emotion";
					var emotionWidth = 3*Math.min(55,emotions[i].emotion-45);
				}else if(emotions[i].emotion>50){
					//yellow to orange
					var emotionColor = 'hsl('+(60-(emotions[i].emotion-50))+',100%,50%)';
					var emotionClass = "bad-emotion";
					var emotionWidth = 3*Math.min(55,emotions[i].emotion-45);
				}else{
					var emotionColor = '#00cc00';
					var emotionClass = "good-emotion";
					var emotionWidth = 3*Math.min(55,55-emotions[i].emotion);
				}

				html += '<td><div class="'+emotionClass+'"" style="width:'+emotionWidth+'px;background-color:'+emotionColor+';"></div></td>';
				html += '</tr>';
			}
		}


		var emotions = [];
		for(var i=0;i<game.crew.length;i++){
			var person = game.crew[i];
			if(!person.equals(this)){
				var emotion = person.getEmotion(this);

				if(emotion >= 60 || emotion <= 40){
					emotions.push({
						person: person,
						emotion: emotion
					});
				}
			}
		}

		var towardsHtml = "";

		if(emotions.length > 0){
			emotions.sort(function(a,b){
				return b.emotion - a.emotion;
			});
			for(var i=0;i<emotions.length;i++){
				if(!emotions[i].person.pc || this.pc){
					towardsHtml += '<tr>';
					towardsHtml += '<td>'+emotions[i].person.name+'</td>';

					if(emotions[i].emotion>=75){
						var emotionColor = '#cc0000';
						var emotionClass = "horrible-emotion";
						var emotionWidth = 3*Math.min(55,emotions[i].emotion-45);
					}else if(emotions[i].emotion>50){
						//yellow to orange
						var emotionColor = 'hsl('+(60-(emotions[i].emotion-50))+',100%,50%)';
						var emotionClass = "bad-emotion";
						var emotionWidth = 3*Math.min(55,emotions[i].emotion-45);
					}else{
						var emotionColor = '#00cc00';
						var emotionClass = "good-emotion";
						var emotionWidth = 3*Math.min(55,55-emotions[i].emotion);
					}

					towardsHtml += '<td><div class="'+emotionClass+'"" style="width:'+emotionWidth+'px;background-color:'+emotionColor+';"></div></td>';
					towardsHtml += '</tr>';
				}
			}
		}

		if(towardsHtml != ""){
			html += '<tr><td colspan="2"><h4>Feelings towards '+this.name+'</h4></td></tr>';
			html += towardsHtml;
		}

		html += '</table>';

		return html;
	}

	hasTrait(traitName){
		for(var i=0;i<this.traits.length;i++){
			if(this.traits[i].name == traitName) return true;
		}
		return false;
	}

	equals(obj){
		return this.id === obj.id;
	}

	getButtonHtml(){

		var status = "";
		if(this.hasTrait("Bleeding")){
			status = "bleeding";
		}else if(this.hasTrait("Infection")){
			status = "infection";
		}else if(this.satiety < 10){
			status = "starving";
		}else{
			var numBitten = this.findExistingSlots().filter(function(s){
				return s.bitten;
			}).length;
			if(numBitten>0){
				status = "bitten";
			}
		}


		var classes = "";
		if(this.isPlayer || this.isOffspring){
			classes+="special-character"
		}
		var html = '<div class="person-button button '+classes+'" button-id="person" object-id="'+this.id+'">';
		html += this.name;
		if(status != ""){
			html += '<div style="color:red">('+
				status +
			')</div>';
		}
		html += '</div>';
		return html;
	}

	getPreviewHtml(fullView, ignoreAttributes){
		if(ignoreAttributes==undefined){
			ignoreAttributes = false;
		}

		var html = '<div class="person-preview">';
		html += '<h3>'+this.name+' - '+this.age+'</h3>';
		html += '<div>'+this.physicalDescription+'</div>';
		if(!ignoreAttributes){
			var that = this;
			var traits = this.traits.filter(function(t){
				return t.name != that.race.name;
			});
			for(var i=0;i<traits.length;i++){
				if(traits[i].display){
					html += traits[i].getHtml();
				}
			}
		}

		if(fullView){
			html += '<div id="person-header-bottom">';
			html += '<div>Speed: '+this.getSpeed()+'</div>';

			var hungerWidth = 3*Math.max(0,this.satiety);
			html += '<div>Satiety: ';
				html += '<div class="hunger-bar" style="width:'+hungerWidth+'px;"></div>';
			html += '</div>';

			html += '<div id="person-buttons" style="text-align: right;">';
				html += '<div class="button" button-id="exile" object-id="'+this.id+'">Exile</div> ';
				html += '<div class="button" button-id="kill" object-id="'+this.id+'">Kill</div> ';
				html += '<div class="button" button-id="give-item" object-id="'+this.id+'">Give item</div> ';
				html += '<div class="button" button-id="fully-equip" object-id="'+this.id+'">Fully Equip</div> ';
			html += '</div>';
		}


		html += '</div>';
		return html;
	}

	getPhysicalDescription(){
		var details = [this.race.name];

		var r1 = Math.random();

		if(Math.random < .2){
			details.push(util.pick([
				"beautiful",
				"good looking",
				"charming",
				"average looking",
				"ugly",
				"wispy",
				"weathered"
			]));
			return util.combineList(details);
		}else{
			if(Math.random()<.4){
				details.push(util.pick([
					"lanky",
					"stout",
					"portly",
					"short",
					"chubby",
					"skinny",
					"average build",
					"tiny",
					"emaciated",
					"bony",
					"stocky",
					"muscular",
					"athletic"
				]));
			}
			if(this.age>50 && Math.random()<.5){
				details.push(util.pick([
					"grey hair",
					"white hair",
					"silver hair"
				]));
			}else if(this.age>20 && Math.random()<.33){
				details.push(util.pick([
					"thinning hair",
					"blue hair",
					"balding",
					"bald"
				]));
			}else{
				details.push(util.pick([
					"black hair",
					"brown hair",
					"red hair",
					"blonde hair",
					"greasy hair",
					"slick hair",
					"straggly hair",
					"full hair",
					"long hair",
					"short hair",
					"curly hair",
					"straight hair",
					"combed hair",
					"cropped hair",
					"messy hair",
					"parted hair",
					"thick hair"
				]));

				if(this.gender == "male" && this.age > 16 && Math.random()<.333){
					details.push(util.pick([	
						"clean shaven",
						"beard",
						"long beard",
						"goatee",
						"mustache",
						"sideburns",
						"stubble"
					]));
				}

				if(Math.random()<.15){
					details.push(util.pick([	
						"narrow face",
						"square face",
						"broad forehead",
						"sharp cheekbones",
						"jutting chin",
						"pointy chin",
						"chin dimple"
					]));
				}

				if(Math.random()<.15){
					details.push(util.pick([	
						"crooked nose",
						"bulbous nose",
						"thin nose",
						"pointed nose",
						"long nose",
						"flared nostrils"
					]));
				}

				if(this.age> 20 && Math.random()<.15){
					details.push(util.pick([	
						"disfigured",
						"scarred",
						"sunburnt",
						"freckles",
						"acne",
						"smooth skin",
						"perfect skin"
					]));
				}

				if(this.age> 25 && Math.random()<.15){
					details.push(util.pick([	
						"missing teeth",
						"crooked teeth",
						"underbite",
						"overbite",
						"yellowed teeth",
						"perfect teeth",
						"white teeth"
					]));
				}


				if(Math.random()<.15){
					details.push(util.pick([
						"tiny eyes",
						"crossed eyed",
						"one eye",
						"sunken eyes",
						"bulging eyes",
						"wide eyes",
						"dull eyes",
						"green eyes",
						"steely eyes",
						"cold eyes",
						"bright eyes"
					]));
				}
			}

			return util.combineList(details);
		}
	}
}