class Tile{
	constructor(x,y, isCamp){
		this.x = x;
		this.y = y;
		this.road = false;
		this.isCamp = isCamp;

		this.people = false;

		this.scavenged = false;

		this.explored = false;

		this.zombies = null;

		if(x % CHUNK_SIZE == 0 || x % CHUNK_SIZE == CHUNK_SIZE-1 ||
		y % CHUNK_SIZE == 0 || y % CHUNK_SIZE == CHUNK_SIZE-1){
			this.roadable = true;
		}else{
			this.roadable = Math.random()<.6
		};

		this.fortifyLevel = 0;

		if(isCamp){
			if(Math.random()<.5){
				this.type = "building";
				this.subtype = 4;
				this.numLayers = numBuildingLayers[this.subtype];
				this.numTypeLayers = 100;
				this.layers = buildingLayers;
			}else{
				this.numLayers = 0;
				this.numTypeLayers = 0;
				this.type = "none";
			}
			this.groundType = "dirt";
		}else{
			if(Math.random()<.4){
				this.numLayers = 0;
				this.numTypeLayers = 0;
				this.type = "none";
			}else{
				if(Math.random()<.95){
					this.type = "tree"
					this.subtype = Math.random()<.5 ? 1 : 0;
					this.numLayers = numTreeLayers;
					this.numTypeLayers = 50;
					this.layers = treeLayers;
				}else{
					this.type = "building";
					var r = Math.random();
					if(r<.1){
						this.fortifyLevel = 3;
						this.subtype = 0;
					}else if(r<.3){
						this.fortifyLevel = 2;
						this.subtype = 1;
					}else if(r<.6){
						this.fortifyLevel = 1;
						this.subtype = 2;
					}else{
						this.fortifyLevel = 1;
						this.subtype = 3;
					}

					this.numLayers = numBuildingLayers[this.subtype];
					this.numTypeLayers = 100;
					this.layers = buildingLayers;
				}
			}

			if(this.isClanLocation()){
				this.fortifyLevel = util.randRange(2,5);
			}

			var r = util.randRange(0,0);
			if(r === 0){
				this.groundType = "grass";
			}else if(r === 1){
				this.groundType = "roadtop";
			}else if(r === 2){
				this.groundType = "roadleft";
			}else if(r === 3){
				this.groundType = "water";
			}
		}

		this.spritesSet = false;

		this.key = x + "," + y;
		map.tiles[this.key] = this;
	}

	isFortified(){
		return this.fortifyLevel >= 2;
	}

	increaseZombies(){
		this.getAllNeighbors().forEach(function(t){
			t.setZombies();
			if(!t.isFortified()){
				//todo: difficulty increase
				if((t.zombies == 0 && !t.equals(camera.getTile()) && Math.random()<.4)
				|| Math.random()<.1){
					t.zombies+=util.randRange(1,Math.floor(2+game.difficultyFactor));
				}
			};
		});
	}

	explore(){
		this.explored = true;

		this.getAllNeighbors().forEach(function(t){
			t.setZombies();
		});

		var r = Math.random();
		var findChance =(this.getPopulationDensity()/game.difficultyFactor)/12;
		if(r<findChance){
			this.people = true;
		}
	}



	scavenge(){
		if(this.isClanLocation() && !this.getClan().dead){
			menu.toast("This area is too heavily guarded to scavenge!");
			game.tick(.25);
		}else if(this.scavenged){
			menu.toast("You already searched this place.");
		}else{
			this.scavenged = true;

			var items = stash.generateItems(this);

			for(var i=0;i<items.length;i++){
				menu.toast("Found "+items[i].getName());
				items[i].putInStash();
			}
			if(items.length == 0){
				menu.toast("There's nothing here.");
			}

			var dt = .5+5/(game.crew.length+2);

			game.tick(dt);
		}
	}

	getItems(numPeople){
		var items = stash.generateItems(this, numPeople);

		for(var i=0;i<items.length;i++){
			menu.toast("Found "+items[i].getName());
			items[i].putInStash();
		}
	}

	getZombies(){
		if(util.distance(this.x,this.y, camera.getTile().x,camera.getTile().y)<2){
			return this.zombies;
		}else{
			return "?";
		}
	}

	getLongDescription(){
		var html = "";
		html += "<strong>Coordinates: </strong> " + this.getLatitude() + " " + this.getLongitude();
		html += "<br>";
		html += "<strong>Area: </strong>" + this.getDescription();
		html += "<br>";
		html += "<strong>Fortifications: </strong>" + this.fortifyLevel;

		var clanTile = this.getClanLocation();
		if(clanTile.equals(this)){
			var localClan = this.getClan();
			if(localClan.revealed){
				html += ' <span style="color:white;"><strong>['+localClan.name.toUpperCase()+']</strong></span>';
			}
		}

		html += "<br>";
		html += "<strong>Zombies: </strong>" + this.getZombies();	
		return html;
	}

	getPopulationDensity(){
		var base = 1;
		if(this.type == "tree"){
			base = 1;
		}else if(this.type == "building"){
			if(this.subtype == 0){
				base = 15;
			}else if(this.subtype == 1){
				base = 8;
			}else if(this.subtype == 2){
				base = 5;
			}else{
				base = 3;
			}
		}else if(this.road){
			base = 2;
		}else{
			base = 1;
		}
		return base;
	}

	setZombies(){
		if(this.zombies === null){
			if(this.isCamp){
				this.zombies = 0;
			}else{
				var base = 2+this.getPopulationDensity();
				var factor = 1;

				this.zombies = base+util.randRange(1,Math.ceil(factor*base));
			}
		}
	}

	getDescription(){
		if(this.type == "tree"){
			return "Forest";
		}else if(this.type == "building"){
			if(this.subtype == 0){
				return "City";
			}else if(this.subtype == 1){
				return "Town";
			}else if(this.subtype == 2){
				return "Suburbs";
			}else if(this.subtype == 3){
				return "Rural";
			}else{
				return "Refugee camp";
			}
		}else if(this.road){
			return "Highway";
		}else{
			return "Field";
		}
	}

	equals(a){
		return this.key == a.key;
	}

	setSprites(){
		if(!this.spritesSet){
			this.groundSprite = PIXI.Sprite.fromImage(dataUrls[images[this.groundType].path]);
			this.outlineSprite = PIXI.Sprite.fromImage(dataUrls[images["outline"].path]);
			this.starSprite = PIXI.Sprite.fromImage(dataUrls[images["star"].path]);
			this.walkSprite = PIXI.Sprite.fromImage(dataUrls[images["walk"].path]);
			this.driveSprite = PIXI.Sprite.fromImage(dataUrls[images["drive"].path]);
			this.spritesSet = true;

			if(this.roadLeft)this.roadLeftSprite = PIXI.Sprite.fromImage(dataUrls[images["roadleft"].path]);
			if(this.roadRight)this.roadRightSprite = PIXI.Sprite.fromImage(dataUrls[images["roadright"].path]);
			if(this.roadTop)this.roadTopSprite = PIXI.Sprite.fromImage(dataUrls[images["roadtop"].path]);
			if(this.roadBottom)this.roadBottomSprite = PIXI.Sprite.fromImage(dataUrls[images["roadbottom"].path]);

			this.layerSprites = [];
			this.shadowSprites = [];

			var numToLoad = this.numLayers;
			if(numToLoad>0){
				numToLoad++;
			}

			for(var i=0;i<numToLoad;i++){
				this.layerSprites[i] = PIXI.Sprite.fromImage(dataUrls[
					this.layers[this.subtype*(this.numTypeLayers+1)+i].path
				]);
				this.shadowSprites[i] = PIXI.Sprite.fromImage(dataUrls[
					this.layers[this.subtype*(this.numTypeLayers+1)+i].path
				]);
			}
		}
	}

	drawGround(){
		var tileX = camera.tileSize*(this.x + camera.getOffsetX());
		var tileY = camera.tileSize*(this.y + camera.getOffsetY());

		var groundSprites = [this.groundSprite];

		if(this.roadLeft) groundSprites.push(this.roadLeftSprite);
		if(this.roadRight) groundSprites.push(this.roadRightSprite);
		if(this.roadTop) groundSprites.push(this.roadTopSprite);
		if(this.roadBottom) groundSprites.push(this.roadBottomSprite);

		if(this.x == camera.x && this.y == camera.y){
			groundSprites.push(this.outlineSprite);
		}

		for(var i=0;i<groundSprites.length;i++){
			groundSprites[i].anchor.set(0);
			groundSprites[i].rotation = 0;
			groundSprites[i].position.x = tileX;
			groundSprites[i].position.y = tileY;
			groundSprites[i].scale = new PIXI.Point(
				camera.tileSize/256,
				camera.tileSize/256
			);

			groundSprites[i].tint = game.timeTint;
			stage.addChild(groundSprites[i]);
		}

		/*ctx.drawImage(
			images[this.groundType].img,
			0,
			0,
			256,
			256,
			tileX,
			tileY,
			camera.tileSize,
			camera.tileSize
		);	*/
		return;
/*
		if(false){
			ctx.drawImage(
				images.selector.img,
				0,
				0,
				256,
				256,
				tileX,
				tileY,
				camera.tileSize,
				camera.tileSize
			);
		}

		if(false){
			ctx.drawImage(
				images.outline.img,
				0,
				0,
				256,
				256,
				tileX,
				tileY,
				camera.tileSize,
				camera.tileSize
			);
		}*/
	}

	drawShadows(){
		if(uglyMode) return;

		for(var k=0;k<this.numLayers;k++){
			var layerIndex = k;

			var expandedSize = camera.tileSize;
			var expandMargin = 0;
			if(this.type === "tree"){
				expandedSize = camera.tileSize * 1.25;
				expandMargin = .125;
			}

			var tileScaling = expandedSize / 256;

			var shadowAmount = game.shadowAmount;
			var shadowAngle = game.shadowAngle;

			var dist = util.distance(camera.x,camera.y,this.x,this.y-.25);

			var tiltX = Math.cos(shadowAngle);
			var tiltY = Math.sin(shadowAngle);

			var tileX = camera.tileSize*(this.x-expandMargin + camera.getOffsetX())+shadowAmount*tileScaling*layerIndex*tiltX*1;
			var tileY = camera.tileSize*(this.y-expandMargin + camera.getOffsetY())+shadowAmount*tileScaling*layerIndex*tiltY*1;

			var sprite = this.shadowSprites[layerIndex];
			sprite.anchor.set(0);
			sprite.rotation = 0;

			sprite.position.x = tileX;
			sprite.position.y = tileY;

			sprite.scale = new PIXI.Point(
				expandedSize/256,
				expandedSize/256
			);

			sprite.tint = 0;

			sprite.alpha = game.shadowAlpha;

			stage.addChild(sprite);
		}
	}

	draw(layerIndex){
		if(uglyMode){
			if(layerIndex>0){
				return;
			}else{
				//last layer is flat
				layerIndex = this.numLayers;
			}
		}else{
			if(layerIndex >= this.numLayers) return;
		}

		if(this.groundType == "water" || this.type === "none") return;
		//bundleId = 0;

		if(layerIndex === 1 && !game.isNight()){
			this.drawShadows();
		}

		var expandedSize = camera.tileSize;
		var expandMargin = 0;
		if(this.type === "tree" && !uglyMode){
			expandedSize = camera.tileSize * 1.25;
			expandMargin = .125;
		}

		var tileScaling = expandedSize / 256;

		//draw layers 1/8 over in each direction

		var angle = Math.atan2(this.y-.25-camera.y, this.x-camera.x);

		var dist = util.distance(camera.x,camera.y,this.x,this.y-.25);

		var tiltX = Math.cos(angle)*Math.min(1,dist/(camera.centerOffsetX));
		var tiltY = Math.sin(angle)*Math.min(1,dist/(camera.centerOffsetX));

		var tileX = camera.tileSize*(this.x-expandMargin + camera.getOffsetX())+tileScaling*layerIndex*tiltX*1;
		var tileY = camera.tileSize*(this.y-expandMargin + camera.getOffsetY())+tileScaling*layerIndex*tiltY*1;

		var sprite = this.layerSprites[layerIndex];
		sprite.anchor.set(0);
		sprite.rotation = 0;

		sprite.position.x = tileX;
		sprite.position.y = tileY;

		sprite.scale = new PIXI.Point(
			expandedSize/256,
			expandedSize/256
		);

		sprite.tint = game.timeTint;

		stage.addChild(sprite);



	}

	drawIcons(){
		var sprite = null;

		var localClan = this.getClan();
		if(localClan.revealed){
			var clanTile = this.getClanLocation();
			if(clanTile.equals(this)){
				var sprite = this.starSprite;

				sprite.scale = new PIXI.Point(
					camera.tileSize/128,
					camera.tileSize/128
				);
			}
		}

		if(camera.getTile().equals(this)){
			if(game.isDriving()){
				var sprite = this.driveSprite;
			}else{
				var sprite = this.walkSprite;
			}

			sprite.scale = new PIXI.Point(
				camera.tileSize/96,
				camera.tileSize/96
			);
		}

		if(sprite != null){
			sprite.anchor.set(.5);
			//sprite.rotation = 0;

			var tileX = camera.tileSize*(this.x +.5+ camera.getOffsetX());
			var tileY = camera.tileSize*(this.y +.5+ camera.getOffsetY());

			sprite.position.x = tileX;
			sprite.position.y = tileY;

			stage.addChild(sprite);
		}
	}

	getClan(){
		return this.getChunk().clan;
	}

	isClanLocation(){
		var chunk = this.getChunk();
		var clanX = chunk.clan.location.x;
		var clanY = chunk.clan.location.y; 
		return this.x == clanX && this.y == clanY;
	}

	getClanLocation(){
		return this.getChunk().getClanTile();
	}

	drawBig(){
		if(uglyMode) return;

		if(bigStage.children.length === 0){
			var add = true;
		}else{
			var add = false;
		}

		this.setSprites();

		var bigX = 145;
		var bigY = 350;

		var groundSprites = [this.groundSprite];


		var tileScaling = .75;

		if(this.roadLeft) groundSprites.push(this.roadLeftSprite);
		if(this.roadRight) groundSprites.push(this.roadRightSprite);
		if(this.roadTop) groundSprites.push(this.roadTopSprite);
		if(this.roadBottom) groundSprites.push(this.roadBottomSprite);

		for(var i=0;i<groundSprites.length;i++){
			groundSprites[i].position.x = bigX;
			groundSprites[i].position.y = bigY;
			groundSprites[i].anchor.set(0.5);
			groundSprites[i].rotation = game.bigAngle;
			groundSprites[i].scale = new PIXI.Point(tileScaling,tileScaling);
			if(add) bigStage.addChild(groundSprites[i]);
		}

		game.bigAngle += .003;
		for(var layerIndex=0;layerIndex<this.numLayers;layerIndex++){

			var expandMargin = 0;


			var tileX = bigX;
			var tileY = bigY-tileScaling*layerIndex*1;
			var sprite = this.layerSprites[layerIndex];

			sprite.position.x = tileX;
			sprite.position.y = tileY;
			sprite.anchor.set(0.5);
			sprite.rotation = game.bigAngle;

			sprite.scale = new PIXI.Point(
				tileScaling,
				tileScaling
			);
			sprite.tint = game.timeTint;

			if(add) bigStage.addChild(sprite);
		}
	}

	getLatitude(){
		return (-this.y*173+357795)/10000;
	}

	getLongitude(){
		return (this.x*173+-786381)/10000;

	}

	getLeftNeighbor(){
		var key = (this.x-1)+","+(this.y);
		return map.tiles[key];
	}

	getRightNeighbor(){
		var key = (this.x+1)+","+(this.y);
		return map.tiles[key];
	}

	getTopNeighbor(){
		var key = (this.x)+","+(this.y-1);
		return map.tiles[key];
	}

	getBottomNeighbor(){
		var key = (this.x)+","+(this.y+1);
		return map.tiles[key];
	}

	getAllNeighbors(){
		var neighbors = [];
		for(var i=-1;i<=1;i++){
			for(var j=-1;j<=1;j++){
				var tile = map.tile(
					camera.getTile().x+i,
					camera.getTile().y+j
				);
				neighbors.push(tile);
			}
		}
		return neighbors;
	}

	getChunk(){
		return map.chunk(
			Math.floor(this.x/CHUNK_SIZE),
			Math.floor(this.y/CHUNK_SIZE)
		);
	}

	getLocalX(){
		var mod = this.x % CHUNK_SIZE;
		if(mod < 0 ) mod += CHUNK_SIZE;
		return mod;
	}

	getLocalY(){
		var mod = this.y % CHUNK_SIZE;
		if(mod < 0) mod += CHUNK_SIZE;
		return mod;
	}
}